"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateQuery = void 0;
const allowed_1 = require("./allowed");
function validateField(key, allowedOperators) {
    // only validate key starting with $ to check of invalid MongoDB uperator
    // eg: $exists is valid $exist is a common typo
    if (key.startsWith('$') && !allowedOperators.includes(key)) {
        return false;
    }
    return true;
}
function traverse(obj, maxDepth = 0, // 0 it's up to infinity
path = '', currentDepth = 0, invalidFields = []) {
    for (const key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            const value = obj[key];
            const fullPath = path ? `${path}.${key}` : key;
            if (typeof value === 'object' && value !== null) {
                if (
                // If 0 it's up to infinity
                maxDepth === 0 ||
                    currentDepth < maxDepth) {
                    traverse(value, maxDepth, fullPath, currentDepth + 1, invalidFields); // recursively recurse nested objects
                }
            }
            else {
                // the first invalid field
                if (!validateField(key, allowed_1.ALLOWED_OPERATORS)) {
                    invalidFields.push(fullPath);
                }
            }
        }
    }
    return {
        isValidQuery: invalidFields.length === 0,
        invalidFields,
    };
}
function validateQuery(obj, 
// Zero is infinity
maxDepth = 0) {
    return traverse(obj, maxDepth);
}
exports.validateQuery = validateQuery;
exports.default = validateQuery;
